'''
Created on 13 déc. 2014

@author: inso
'''
import unittest
from ucoinpy.documents.status import Status

raw_status = """Version: 1
Type: Status
Currency: beta_brousouf
Status: UP
Block: 8-1922C324ABC4AF7EF7656734A31F5197888DDD52
From: HsLShAtzXTVxeUtQd7yi5Z5Zh4zNvbu8sTEZ53nfKcqY
To: 8Fi1VSTbjkXguwThF4v2ZxC5whK7pwG2vcGTkPUPjPGU
dkaXIiCYUJtCg8Feh/BKvPYf4uFH9CJ/zY6J4MlA9BsjmcMe4YAblvNt/gJy31b1aGq3ue3h14mLMCu84rraDg==
"""


class Test_Status(unittest.TestCase):
    def test_fromraw(self):
        status = Status.from_signed_raw(raw_status)
        self.assertEqual(status.status, 'UP')
        self.assertEqual(status.blockid, "8-1922C324ABC4AF7EF7656734A31F5197888DDD52")
        self.assertEqual(status.sender, "HsLShAtzXTVxeUtQd7yi5Z5Zh4zNvbu8sTEZ53nfKcqY")
        self.assertEqual(status.recipient, "8Fi1VSTbjkXguwThF4v2ZxC5whK7pwG2vcGTkPUPjPGU")
        self.assertEqual(status.signatures[0], "dkaXIiCYUJtCg8Feh/BKvPYf4uFH9CJ/zY6J4MlA9BsjmcMe4YAblvNt/gJy31b1aGq3ue3h14mLMCu84rraDg==")

    def test_fromraw_toraw(self):
        status = Status.from_signed_raw(raw_status)
        rendered_status = status.signed_raw()
        from_rendered_status = Status.from_signed_raw(rendered_status)
        self.assertEqual(from_rendered_status.status, 'UP')
        self.assertEqual(from_rendered_status.blockid, "8-1922C324ABC4AF7EF7656734A31F5197888DDD52")
        self.assertEqual(from_rendered_status.sender, "HsLShAtzXTVxeUtQd7yi5Z5Zh4zNvbu8sTEZ53nfKcqY")
        self.assertEqual(from_rendered_status.recipient, "8Fi1VSTbjkXguwThF4v2ZxC5whK7pwG2vcGTkPUPjPGU")
        self.assertEqual(from_rendered_status.signatures[0], "dkaXIiCYUJtCg8Feh/BKvPYf4uFH9CJ/zY6J4MlA9BsjmcMe4YAblvNt/gJy31b1aGq3ue3h14mLMCu84rraDg==")




"""
Created on 2 déc. 2014

@author: inso
"""
from .. import PROTOCOL_VERSION
from .document import Document

import re


class Membership(Document):
    """
.. note:: A membership document is specified by the following format :

    | Version: VERSION
    | Type: Membership
    | Currency: CURRENCY_NAME
    | Issuer: ISSUER
    | Block: NUMBER-HASH
    | Membership: MEMBERSHIP_TYPE
    | UserID: USER_ID
    | CertTS: CERTIFICATION_TS

    """

    # PUBLIC_KEY:SIGNATURE:NUMBER:HASH:TIMESTAMP:USER_ID
    re_inline = re.compile("([1-9A-Za-z][^OIl]{42,45}):([A-Za-z0-9+/]+(?:=|==)?):\
([0-9]+):([0-9a-fA-F]{5,40}):([0-9]+):([^\n]+)\n")
    re_type = re.compile("Type: (Membership)")
    re_issuer = re.compile("Issuer: ([1-9A-Za-z][^OIl]{42,45})\n")
    re_block = re.compile("Block: ([0-9]+)-([0-9a-fA-F]{5,40})\n")
    re_membership_type = re.compile("Membership: (IN|OUT)")
    re_userid = re.compile("UserID: ([^\n]+)\n")
    re_certts = re.compile("CertTS: ([0-9]+)\n")



    def __init__(self, version, currency, issuer, block_number, block_hash,
                 membership_type, uid, cert_ts, signature):
        """
        Constructor
        """
        super().__init__(version, currency, [signature])
        self.issuer = issuer
        self.block_number = block_number
        self.block_hash = block_hash
        self.membership_type = membership_type
        self.uid = uid
        self.cert_ts = cert_ts

    @classmethod
    def from_inline(cls, version, currency, membership_type, inline):
        data = Membership.re_inline.match(inline)
        issuer = data.group(1)
        signature = data.group(2)
        block_number = int(data.group(3))
        block_hash = data.group(4)
        cert_ts = int(data.group(5))
        uid = data.group(6)
        return cls(version, currency, issuer, block_number,
                   block_hash, membership_type, uid, cert_ts, signature)

    @classmethod
    def from_signed_raw(cls, raw, signature=None):
        lines = raw.splitlines(True)
        n = 0

        version = int(Membership.re_version.match(lines[n]).group(1))
        n = n + 1

        Membership.re_type.match(lines[n]).group(1)
        n = n + 1

        currency = Membership.re_currency.match(lines[n]).group(1)
        n = n + 1

        issuer = Membership.re_issuer.match(lines[n]).group(1)
        n = n + 1

        blockid = Membership.re_block.match(lines[n])
        blocknumber = int(blockid.group(1))
        blockhash = blockid.group(2)
        n = n + 1

        membership_type = Membership.re_membership_type.match(lines[n]).group(1)
        n = n + 1

        uid = Membership.re_userid.match(lines[n]).group(1)
        n = n + 1

        cert_ts = int(Membership.re_certts.match(lines[n]).group(1))
        n = n + 1

        signature = Membership.re_signature.match(lines[n]).group(1)
        n = n + 1

        return cls(version, currency, issuer, blocknumber, blockhash,
                   membership_type, uid, cert_ts, signature)

    def raw(self):
        return """Version: {0}
Type: Membership
Currency: {1}
Issuer: {2}
Block: {3}-{4}
Membership: {5}
UserID: {6}
CertTS: {7}
""".format(self.version,
                      self.currency,
                      self.issuer,
                      self.block_number, self.block_hash,
                      self.membership_type,
                      self.uid,
                      self.cert_ts)

    def inline(self):
        return "{0}:{1}:{2}:{3}:{4}:{5}".format(self.issuer,
                                        self.signatures[0],
                                        self.block_number,
                                        self.block_hash,
                                        self.cert_ts,
                                        self.uid)



import re
from .document import Document


class Status(Document):
    """
.. note:: A status document is specified by the following format :

    | Version: VERSION
    | Type: Status
    | Currency: CURRENCY_NAME
    | Status: STATUS
    | Block: BLOCK
    | From: SENDER
    | To: RECIPIENT

    """

    re_type = re.compile("Type: (Status)")
    re_status = re.compile("Status: (NEW|NEW_BACK|UP|UP_BACK|DOWN)")
    re_block = re.compile("Block: ([0-9]+-[0-9a-fA-F]{5,40})\n")
    re_from = re.compile("From: ([1-9A-Za-z][^OIl]{42,45})\n")
    re_to = re.compile("To: ([1-9A-Za-z][^OIl]{42,45})\n")

    def __init__(self, version, currency, status, blockid, sender,
                 recipient, signature):
        """
        Constructor
        """
        super().__init__(version, currency, [signature])

        self.status = status
        self.blockid = blockid
        self.sender = sender
        self.recipient = recipient

    @classmethod
    def from_signed_raw(cls, raw):
        lines = raw.splitlines(True)
        n = 0

        version = int(Status.re_version.match(lines[n]).group(1))
        n = n + 1

        Status.re_type.match(lines[n]).group(1)
        n = n + 1

        currency = Status.re_currency.match(lines[n]).group(1)
        n = n + 1

        status = Status.re_status.match(lines[n]).group(1)
        n = n + 1

        blockid = Status.re_block.match(lines[n]).group(1)
        n = n + 1

        sender = Status.re_from.match(lines[n]).group(1)
        n = n + 1

        recipient = Status.re_to.match(lines[n]).group(1)
        n = n + 1

        signature = Status.re_signature.match(lines[n]).group(1)
        n = n + 1

        return cls(version, currency, status, blockid,
                   sender, recipient, signature)

    def raw(self):
        return """Version: {0}
Type: Status
Currency: {1}
Status: {2}
Block: {3}
From: {4}
To: {5}
""".format(self.version, self.currency, self.status,
           self.blockid, self.sender, self.recipient)



import unittest
from ucoinpy.api.bma import API
from ucoinpy.documents.peer import BMAEndpoint


class Test_BMA_API(unittest.TestCase):

    def test_reverse_url_complete(self):
        endpoint = BMAEndpoint("test.com", "124.2.2.1", "2001:0db8:0000:85a3:0000:0000:ac1f:8001 ", 9092)
        api = API(endpoint.conn_handler(), "any")
        self.assertEqual(api.reverse_url("/test/url"), "http://test.com:9092/any/test/url")

    def test_reverse_url_only_ipv4(self):
        endpoint = BMAEndpoint(None, "124.2.2.1", None, 9092)
        api = API(endpoint.conn_handler(), "any")
        self.assertEqual(api.reverse_url("/test/url"), "http://124.2.2.1:9092/any/test/url")

    def test_reverse_url_only_ipv6(self):
        endpoint = BMAEndpoint(None, None, "2001:0db8:0000:85a3:0000:0000:ac1f:8001", 9092)
        api = API(endpoint.conn_handler(), "any")
        self.assertEqual(api.reverse_url("/test/url"), "http://2001:0db8:0000:85a3:0000:0000:ac1f:8001:9092/any/test/url")
